// Example sketch for interfacing with the DS1302 timekeeping chip.
//
// Copyright (c) 2009, Matt Sparks
// All rights reserved.
//
// Changed 2018 by sysconsol: time without dayname, therefore with number for the day
// 1 -> Sunday
// 2 -> Monday
// ...
// 7 -> Saturday
//
// Changed 2018 by sysconsol: set time via terminal (not failsafe!!)
//
// http://quadpoint.org/projects/arduino-ds1302
#include <stdio.h>
#include <DS1302.h>

namespace {

// Set the appropriate digital I/O pin connections. These are the pin
// assignments for the Arduino as well for as the DS1302 chip. See the DS1302
// datasheet:
//
//   http://datasheets.maximintegrated.com/en/ds/DS1302.pdf
const int kCePin   = 8;  // Chip Enable
const int kIoPin   = 9;  // Input/Output
const int kSclkPin = 10;  // Serial Clock

// Create a DS1302 object.
DS1302 rtc(kCePin, kIoPin, kSclkPin);

String temp;
String set = "set";
uint16_t jahr = 0;
uint8_t monat = 0;
uint8_t tag = 0;
uint8_t stunde = 0;
uint8_t minute = 0;
uint8_t sekunde = 0;
uint8_t wochentag = 1;
boolean halten;



void printTime() {
  // Get the current time and date from the chip.
  Time t = rtc.time();

  // Format the time and date and insert into the temporary buffer.
  char buf[50];
  snprintf(buf, sizeof(buf), "%02d %04d-%02d-%02d %02d:%02d:%02d",
           t.day,
           t.yr, t.mon, t.date,
           t.hr, t.min, t.sec);

  // Print the formatted string to serial so we can see the time.
  Serial.println(buf);
}

}  // namespace

void setup() {
  Serial.begin(9600);

  // Initialize a new chip by turning off write protection and clearing the
  // clock halt flag. These methods needn't always be called. See the DS1302
  // datasheet for details.
  rtc.writeProtect(false);

  // Make a new time object to set the date and time.
  // Sunday, September 22, 2013 at 01:38:50.
  //  Time t(2013, 9, 22, 1, 38, 50, 01);

  // Set the time and date on the chip.
  //  rtc.time(t);

Serial.println("set time (turn on \\n on enter):set YYYY MM DD HH MM SS Daynumber");
}

// Loop and print the time every second.
void loop() {
  if(halten == false){
    printTime();
    delay(1000);
  }
  // read in serial command
  while (Serial.available() > 0) {
    temp = Serial.readStringUntil("\n");
  }

  // check for the right command (if you use more than one)
  if (temp.startsWith(set)){
    // pause RTC
    rtc.halt(true);
    // pause writing the time to terminal
    halten = true;

    Serial.println(temp); // show input, there must be one free line after! If not check \n on enter
    // cut input string and convert to uint8_t
    jahr = temp.substring(4, 8).toInt();
    monat = temp.substring(9, 11).toInt();
    tag = temp.substring(12, 14).toInt();
    stunde = temp.substring(15, 17).toInt();
    minute = temp.substring(18, 20).toInt();
    sekunde = temp.substring(21, 23).toInt();
    wochentag = temp.substring(24, 26).toInt(); 

    // bring together
    Time t(jahr, monat, tag, stunde, minute, sekunde, wochentag);
    // set the RTC
    rtc.time(t);

    // start the RTC
    rtc.halt(false);
    halten = false;
    // show the uint8_t for debug (or understanding, what is goint on here ;-) )
    Serial.println(jahr);
    Serial.println(monat);
    Serial.println(tag);
    Serial.println(stunde);
    Serial.println(minute);
    Serial.println(sekunde);
    Serial.println(wochentag);
    // clear the string, otherwise the set command will be repeated again and again
    temp = "0";
  }
}
